/*
	Execute a command on the back-end
*/
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include "litefs.h"

static bool debug=false;

static void n_debug (const char *ctl, ...)
{
	if (debug){
		va_list list;
		va_start (list,ctl);
		vfprintf (stderr,ctl,list);
		va_end (list);
	}
}


static void usage()
{
	fprintf (stderr,"litefs-exec version 1.0\n"
		"\n"
		"litefs-exec [ --debug ] --sockets unix_socket command [ args ...]\n");
}

static int litefsexec_loop (int fd)
{
	fd_set fdin;
	FD_ZERO (&fdin);
	FD_SET (0,&fdin);
	FD_SET (fd,&fdin);
	while (1){
		fd_set in;
		memcpy (&in,&fdin,sizeof(in));
		n_debug ("select maxfd %d\n",fd+1);
		if (select (fd+1,&in,NULL,NULL,NULL)!=-1){
			if (FD_ISSET (fd,&in)){
				char buf[32000];
				int n = read (fd,buf,32000);
				if (n == 0){
					break;
				}else{
					write (1,buf,n);
				}
			}
			if (FD_ISSET (0,&in)){
				char buf[32000];
				int n = read (0,buf,32000);
				if (n == 0){
					break;
				}else{
					write (fd,buf,n);
				}
			}
		}
	}
	return 0;
}

static int litefs_exec (const char *sockpath, char *arg[])
{
	int ret = -1;
	int fd =  socket (AF_UNIX,SOCK_STREAM,0);
	if (fd == -1){
		perror("socket client");
	}else{
		struct sockaddr_un un;
		un.sun_family = AF_UNIX;
		strcpy (un.sun_path,sockpath);
		int s = connect(fd,(struct sockaddr*)&un,sizeof(un));
		if (s == -1){
			perror("connect");
		}else{
			LITE_REQUEST_DATA req;
			req.cmd = LITE_EXEC;
			req.mode = 0;
			req.perm = 0;
			req.buf[0] = '\0';
			for (int i=0; arg[i] != NULL; i++){
				if (i > 0) strcat (req.buf," ");
				strcat (req.buf,arg[i]);
			}
			req.len  = strlen(req.buf)+1;
			int ret = write (fd,&req,sizeof(LITE_REQUEST));
			LITE_REPLY rep;
			int len = read (fd,&rep,sizeof(rep));
			if (len == sizeof(LITE_REPLY)){
				if (rep.status == LITE_STATUS_OK){
					ret = litefsexec_loop (fd);
				}else{
				}
			}else{
			}
		}
		close (fd);
	}
	return ret;
}


int main (int argc, char *argv[])
{
	int ret = -1;
	if (argc == 1){
		usage();
	}else{
		const char *sock = NULL;
		int i;
		for (i=1; i<argc; i++){
			if (strcmp(argv[i],"--sockets")==0){
				i++;
				sock = argv[i];
			}else if (strcmp(argv[i],"--debug")==0){
				debug = true;
			}else{
				usage();
			}
		}
		if (sock == NULL | i == argc-1){
			usage();
		}else{
			ret = litefs_exec (sock, argv+i);
		}
	}
	return ret;
}	


